#!/bin/env Rscript

#module load gcc/7.1.0-fasrc01 R/3.5.0-fasrc02

source("https://bioconductor.org/biocLite.R")

library(dada2)
library(reshape2)
library(ggplot2)
library(vegan)

dans_colors <- c("#FF0000","#F6A300","#0068CC","#6600AA","#AC0088","#AA33FF","#00FFFF","#00CC00","#006611","#00AC99",
				 "#AC6844","#FFFF00",
				 "#991100","#ACAC11","#a0f0aa","#FF00FF","#FF8611","#B9F6F6","#001166","#AC9A00","#994141","#ff1169",
				 "#0AF622","#119924","#Ac3311","#004A9A","#AcAc99","turquoise","tomato","sienna1","rosybrown",
				 "peachpuff","olivedrab3","mistyrose1","mediumorchid","indianred2","#114914","#660011","ivory3","deeppink","#331111")


# This block ran on cluster
########
# wd should be ~/g4/fatma_arcella/
setwd('~/g4/fatma_arcella/')
# folder with raw fastqs
raw_path <- 'raw_fastq'

# get the paths by R1/R2
raw_R1 <- sort(list.files(raw_path, pattern="_R1.fastq.gz", full.names = T))
raw_R2 <- sort(list.files(raw_path, pattern="_R2.fastq.gz", full.names = T))

# wrangle out sample names
samps <- gsub("_L001.*$", "", basename(raw_R1))

# set up paths for filtering output
filtN_R1 <- file.path(raw_path, "filtered", paste0(samps, "_R1.filtN.fastq.gz"))
filtN_R2 <- file.path(raw_path, "filtered", paste0(samps, "_R2.filtN.fastq.gz"))

prefilt <- filterAndTrim(raw_R1, filtN_R1, raw_R2, filtN_R2, maxN = 0, compress=T, multithread = F, n = 1e4)


### Check on primers
FWD <- "GTGYCAGCMGCCGCGGTAA" # emp v4 f
REV <- "GGACTACNVGGGTWTCTAAT" # emp v4 r

allOrients <- function(primer) {
	# Create all orientations of the input sequence
	require(Biostrings)
	dna <- DNAString(primer)  # The Biostrings works w/ DNAString objects rather than character vectors
	orients <- c(Forward = dna, Complement = complement(dna), Reverse = reverse(dna),
				 RevComp = reverseComplement(dna))
	return(sapply(orients, toString))  # Convert back to character vector
}
FWD.orients <- allOrients(FWD)
REV.orients <- allOrients(REV)
FWD.orients

primerHits <- function(primer, fn) {
	# Counts number of reads in which the primer is found
	nhits <- vcountPattern(primer, sread(readFastq(fn)), fixed = FALSE)
	return(sum(nhits > 0))
}
rbind(FWD.ForwardReads = sapply(FWD.orients, primerHits, fn = filtN_R1[[1]]),
	  FWD.ReverseReads = sapply(FWD.orients, primerHits, fn = filtN_R2[[1]]),
	  REV.ForwardReads = sapply(REV.orients, primerHits, fn = filtN_R1[[1]]),
	  REV.ReverseReads = sapply(REV.orients, primerHits, fn = filtN_R2[[1]]))
# only get 87F,255R readthrough, RC/core clearly trimmed the end but not the readthroughs

cutadapt <- "/Users/drutter/virtual-envs/anvio-stable/bin/cutadapt"
#system2(cutadapt, args = "--version")  # check it

path.cut <- file.path("readthrough_primer_trimmed")
if(!dir.exists(path.cut)) dir.create(path.cut)
filtNcut_R1 <- file.path(path.cut, basename(filtN_R1))
filtNcut_R2 <- file.path(path.cut, basename(filtN_R2))

FWD.RC <- dada2:::rc(FWD)
REV.RC <- dada2:::rc(REV)
# Trim FWD and the reverse-complement of REV off of R1 (forward reads)
R1.flags <- paste("-g", FWD, "-a", REV.RC)
# Trim REV and the reverse-complement of FWD off of R2 (reverse reads)
R2.flags <- paste("-G", REV, "-A", FWD.RC)
# Run Cutadapt
for(i in seq_along(filtN_R1)) {
	system2(cutadapt, args = c(R1.flags, R2.flags, "-n", 2, # -n 2 required to remove FWD and REV from reads
							   "-o", filtNcut_R1[i], "-p", filtNcut_R2[i], # output files
							   filtN_R1[i], filtN_R2[i])) # input files
}

# quick check
rbind(FWD.ForwardReads = sapply(FWD.orients, primerHits, fn = filtNcut_R1[[6]]),
	  FWD.ReverseReads = sapply(FWD.orients, primerHits, fn = filtNcut_R2[[6]]),
	  REV.ForwardReads = sapply(REV.orients, primerHits, fn = filtNcut_R1[[6]]),
	  REV.ReverseReads = sapply(REV.orients, primerHits, fn = filtNcut_R2[[6]]))

# DONE - proceeding onwards

# filter and trim
# note that truncLen (length to truncate each to) is set to 0 - NOT TRUNCATING arbitrarily, only based on Q
filtered <- "filtered"
filtAll_R1 <- file.path(filtered, basename(filtNcut_R1))
filtAll_R2 <- file.path(filtered, basename(filtNcut_R2))
out <- filterAndTrim(filtNcut_R1, filtAll_R1, filtNcut_R2, filtAll_R2, minLen = 50, maxN = 0, maxEE=c(2,2), truncQ=2, rm.phix=T, compress=T, multithread=F, n = 1e4)


err_R1 <- learnErrors(filtAll_R1, multithread=T)  # increase nbases param to model more accurately?
err_R2 <- learnErrors(filtAll_R2, multithread=T)
# would be wise to visualize error rates here...
plotErrors(err_R1, nominalQ=TRUE)
plotErrors(err_R2, nominalQ=TRUE)

# dereplicate step. here many are on the order of 1/4 reads are unique, but there are noticeable 3/4 reads are unique
derep_R1 <- derepFastq(filtAll_R1, verbose=T)
derep_R2 <- derepFastq(filtAll_R2, verbose=T)

# save point
#save.image()


# get the ASVs - not pooling so each sample is independent - potentially add "pool = T" and compare
dada_R1 <- dada(derep_R1, err=err_R1, multithread = T)
dada_R2 <- dada(derep_R2, err=err_R2, multithread = T)


#save.image()

# merge it
merged <- mergePairs(dada_R1, derep_R1, dada_R2, derep_R2, verbose = T)

# get that beloved OTU table
arcella <- makeSequenceTable(merged)


# drop chimeras
arcella_data <- removeBimeraDenovo(arcella, method = "consensus", multithread=T, verbose=T)


print("DONE AS PLANNED")
#save.image()

######



###################
# everytime stuff #
###################

# load odyssey crunch output to local
#load("~/g4/joey_cave/odyssey_20180920.RData")

# define some colors early on for consistency
sampColors <- setNames(rep("black", length(samps)), samps)
sampColors[grep("water", names(sampColors), ignore.case = T)] <- 'blue'
sampColors[grep("neg", names(sampColors), ignore.case = T)] <- 'grey66'
sampColors[grep("remipede", names(sampColors), ignore.case = T)] <- 'red'




#########################
# summary plots from QC through dada function
########################

# Survivorship through filtering step (no trunctation)

plot(out[,"reads.in"], out[,"reads.out"], xlab = "Raw reads (count)", ylab = "Reads surviving filtration",
	 main = "Filtration survivorship\npoints are samples", pch = 16, col = sampColors)
lines(x = c(0,max(out[,2])), y = c(0, max(out[,2])), lty = 2)
legend("topleft", legend = c("Sample", "Water", "NegControl", "Remipede"), fill = c("black","blue","grey66","red"))

# what does the distribution look like
boxplot(out[,2]/out[,1], ylab = "Fraction Kept After QC", pch = 16)

# print out underperforming samples = where an excess portion got dropped
underperformThresh <- 0.66
out[out[,2] <= (out[,1]*underperformThresh),]
# fresh colors
underperformCols <- sampColors
underperformCols[out[,2] <= (out[,1]*underperformThresh)] <- 'orange'
# highlight them
plot(out[,"reads.in"], out[,"reads.out"], xlab = "Raw reads (count)", ylab = "Reads surviving filtration",
	 main = "Filtration survivorship\npoints are samples", pch = 16, col = underperformCols)
lines(x = c(0,max(out[,2])), y = c(0, max(out[,2])), lty = 2)
lines(x = c(0,max(out[,2])), y = c(0, underperformThresh*max(out[,2])), lty = 2, col = 'orange')
legend("topleft", legend = c("Sample", "Water", "NegControl", "Remipede", paste0("<= ",underperformThresh*100,"% kept")),
	   fill = c("black","blue","grey66","red","orange"))





# what does error model look like?
plotErrors(err_R1, nominalQ = T)   # looks OK - not liking the double peaks on some of the estimates, but it's a decent fit





# track reads at key points - copied from dada2 tutorial
getN <- function(x) sum(getUniques(x))
track_wide <- cbind(out, sapply(dada_R1, getN), sapply(dada_R2, getN), sapply(merged, getN), rowSums(arcella_data))
colnames(track_wide) <- c("input", "filtered", "denoisedF", "denoisedR", "merged", "nonchim")
rownames(track_wide) <- samps
track_wide <- data.frame(track_wide)
track_wide$sample <- factor(rownames(track_wide), levels = names(sampColors))
track <- melt(track_wide)
colnames(track)[2:3] <- c("step", "reads")

# visualize
# sum all
pdf("~/g4/fatma_arcella/reads_through_dada_all_with_controls.pdf", width = 5, height = 6)
ggplot(track, aes(x = step, y = reads)) +
	geom_bar(aes(fill = step), stat = 'identity') + theme_minimal() +
	theme(axis.text.x = element_text(angle = 90), panel.background = element_blank(), axis.line.x = element_line(), axis.line.y = element_line())
dev.off()

# max granularity
pdf("~/g4/fatma_arcella/per_sample_dada_read_tracking_with_controls.pdf", width = 9, height = 8)
ggplot(track, aes(x = step, y = reads)) +
	geom_bar(stat = 'identity', aes(fill = sample), position = 'identity', show.legend = F) +
	scale_x_discrete(expand = c(0,0)) + scale_y_continuous(expand = c(0,0)) +
	facet_wrap(~sample, ncol = 6, scales = 'fixed') +
	scale_fill_manual(values = underperformCols) + theme_minimal() +
	theme(axis.text.x = element_text(angle = 90), panel.background = element_blank(), axis.line.x = element_line(), axis.line.y = element_line(),
		  axis.text.y = element_text(size = 2), panel.grid = element_blank())
dev.off()

##########################
# INSPECT PROCESSED DATA
##########################

# some nice functions for later
subsetRelAbund <- function(df, column = 'sample', toKeep) {
	if(is.numeric(toKeep)) {
		toKeep <- unique(as.character(df[,column]))[1:toKeep]
	}

	df <- df[which(df[,column] %in% toKeep),]
	df <- droplevels(df)

	return(df)
}

# make nice counts matrix, rename ASVs to arbitrary designations, decreasing order
arcella_counts <- as.data.frame(arcella_data)
arcella_counts <- arcella_counts[,rev(order(colSums(arcella_counts)))]
asvsToSeqDict <- setNames(sprintf("ASV_%04d", 1:ncol(arcella_counts)), colnames(arcella_counts))
## SAVE THE SEQUENCES AND NEW IDS!!
write.table(matrix(c(asvsToSeqDict, names(asvsToSeqDict)), ncol = 2, byrow=F),
			file = "~/g4/fatma_arcella/asv_sequences.txt", col.names = F, row.names = F, quote = F, sep = "\t")
colnames(arcella_counts) <- asvsToSeqDict
rownames(arcella_counts) <- samps
write.table(arcella_counts, file = "~/g4/fatma_arcella/asv_counts.txt", col.names = T, row.names = T, quote = F, sep = "\t")


# distribution of ASVs
barplot(colSums(arcella_counts), xlab = 'ASVs', ylab = 'Reads (count)', axisnames = F)
text(x = 6000, y = 3.2e5, labels = paste0(sum(1*(colSums(arcella_counts) < 500))," ASVs with\n< 500 reads"))

# samples reads
barplot(rev(sort(rowSums(arcella_counts))), col = underperformCols[rev(order(rowSums(arcella_counts)))], xlab = 'Samples', ylab = 'Reads (count)', axisnames = F)
text(x = 200, y = 9e4, labels = paste0(sum(1*(rowSums(arcella_counts) < 5000))," samples with\n< 5,000 reads"))
text(x = 200, y = 7e4, labels = paste0(sum(1*(rowSums(arcella_counts) == 0))," samples with\n exactly 0 reads"))
title(paste0("Mean reads per sample:",round(mean(rowSums(arcella_counts)),0)))

